#include "car.h"

#define COEFF 1

void move_car(struct _car *car,int keys,SDL_Surface *fun)
{
  Uint32 c;
  Uint8 r,g,b;
  
  /* reset flags */
  car->crashflag=0;
  
  /* get the pixel color under the center of car in the function map */
  c=getpixel(fun,car->x,car->y);
  /* red layer (checkpoints) */
  /* green layer (road quality) */
  /* blue layer (grip) */
  SDL_GetRGB(c,fun->format,&r,&g,&b);

  if (keys & 8) /* up */
    car->speed+=0.01*2*COEFF;
  if (keys & 4) /* down */
    car->speed-=0.01*COEFF;
  if (keys & 2) /* left */
  {
    if (car->speed<0)
      car->angle+=0.01*(255-b)/255*COEFF;
    else
      car->angle-=0.01*(255-b)/255*COEFF;
  }
  if (keys & 1) /* right */
  {
    if (car->speed<0)
      car->angle-=0.01*(255-b)/255*COEFF;
    else
      car->angle+=0.01*(255-b)/255*COEFF;
  }
  
  /* limit angle between 0 and 2*pi */
  if (car->angle<0) car->angle+=2*M_PI;
  if (car->angle>2*M_PI) car->angle-=2*M_PI;
  
  /* update the speed depending on the road quality */
  car->speed-=car->speed*(255-g)/1000;
 
  /* if it is a wall we move back to the last position */
  if (g==0)
  {
    car->x=car->ox;
    car->y=car->oy;
    car->crashflag=1;
  }
  
  /* save the old position and compute the new one */
  car->ox=car->x;
  car->oy=car->y;
  car->speed*=0.995;
  car->x-=cos(car->angle)*car->speed;
  car->y-=sin(car->angle)*car->speed;
  
  /* collision with the border of the screen */
  if (car->x<car->w/2 || car->x>fun->w-car->w/2 || car->y<car->h/2 || car->y>fun->h-car->h/2)
  {
    car->x=car->ox;
    car->y=car->oy;
    car->speed=0;
    car->crashflag=1;
  }
  
  /* if we are on the next checkpoint, validate it */
  if (r/8==car->lastcheck+1)
  {
    /* If we validate a missed checkpoint */
    if (car->lapflag==3) car->lapflag=4;
    car->lastcheck++;
  }
  
  /* if we missed a checkpoint */
  if ((r/8>car->lastcheck+1) && (car->lastcheck!=0)) car->lapflag=3;

  /* if we validate all and start over, we complete a turn */
  if (r/8==0 && car->lastcheck==31)
  {
    /* reset turn variables */
    car->lastcheck=0;
    car->lap++;
    car->lapflag=1;
  }
  
  /* if we are at the start but not each checkpoint validate, it's an incomplete lap */
  if (r/8==0 && r!=0 && car->lastcheck!=31 && car->lastcheck>0)
  {
    car->lastcheck=0;
    car->lapflag=2;
  }
  
  /* if the car is braking, display red lights */
  if (keys & 4 && car->speed>0.1) car->lights_brake=1; else car->lights_brake=0;
  
  /* if the car is going backwards, display white lights */
  if (car->speed<-0.1) car->lights_backwards=1; else car->lights_backwards=0;
  
  /* if the car is stopped, then warning */
  if (car->speed>=-0.1 && car->speed<=0.1)
    car->lights_warning=1;
  else
    car->lights_warning=0;
  
  return;
}
